@@ -8,6 +8,8 @@ Simple authentication with OAuth.
 
 import json
 import os
+import warnings
+from pathlib import Path
 
 from google.oauth2.credentials import Credentials
 from google.oauth2.service_account import Credentials as ServiceAccountCredentials
@@ -35,20 +37,30 @@ def get_config_dir(config_dir_name="gspread", os_is_windows=os.name == "nt"):
 
     """
     if os_is_windows:
-        return os.path.join(os.environ["APPDATA"], config_dir_name)
+        return Path(os.environ["APPDATA"], config_dir_name)
     else:
-        return os.path.join(os.path.expanduser("~"), ".config", config_dir_name)
+        return Path(Path.home(), ".config", config_dir_name)
 
 
 DEFAULT_CONFIG_DIR = get_config_dir()
 
-DEFAULT_CREDENTIALS_FILENAME = os.path.join(DEFAULT_CONFIG_DIR, "credentials.json")
-DEFAULT_AUTHORIZED_USER_FILENAME = os.path.join(
-    DEFAULT_CONFIG_DIR, "authorized_user.json"
-)
-DEFAULT_SERVICE_ACCOUNT_FILENAME = os.path.join(
-    DEFAULT_CONFIG_DIR, "service_account.json"
-)
+DEFAULT_CREDENTIALS_FILENAME = DEFAULT_CONFIG_DIR / "credentials.json"
+DEFAULT_AUTHORIZED_USER_FILENAME = DEFAULT_CONFIG_DIR / "authorized_user.json"
+DEFAULT_SERVICE_ACCOUNT_FILENAME = DEFAULT_CONFIG_DIR / "service_account.json"
+
+
+def authorize(credentials, client_factory=Client):
+    """Login to Google API using OAuth2 credentials.
+    This is a shortcut/helper function which
+    instantiates a client using `client_factory`.
+    By default :class:`gspread.Client` is used (but could also use
+    :class:`gspread.BackoffClient` to avoid rate limiting).
+
+    :returns: An instance of the class produced by `client_factory`.
+    :rtype: :class:`gspread.client.Client`
+    """
+
+    return client_factory(auth=credentials)
 
 
 def local_server_flow(client_config, scopes, port=0):
@@ -56,7 +68,7 @@ def local_server_flow(client_config, scopes, port=0):
 
     Creates an OAuth flow and runs `google_auth_oauthlib.flow.InstalledAppFlow.run_local_server <https://google-auth-oauthlib.readthedocs.io/en/latest/reference/google_auth_oauthlib.flow.html#google_auth_oauthlib.flow.InstalledAppFlow.run_local_server>`_.
     This will start a local web server and open the authorization URL in
-    the user’s browser.
+    the user's browser.
 
     Pass this function to ``flow`` parameter of :meth:`~gspread.oauth` to run
     a local server flow.
@@ -73,20 +85,32 @@ def console_flow(client_config, scopes):
     Pass this function to ``flow`` parameter of :meth:`~gspread.oauth` to run
     a console strategy.
     """
+    # The console flow has been disabled by google
+    # see: https://developers.googleblog.com/2022/02/making-oauth-flows-safer.html?m=1#disallowed-oob
+    # now: warn the user about this
+    # next: remove this deprecated method
+    warnings.warn(
+        "The gspread.auth.console_flow flow won't succeed.\n"
+        "The OAuth out-of-band (OOB) flow is deprecated. "
+        "New clients will be unable to use this flow starting on Feb 28, 2022. "
+        "This flow will be deprecated for all clients on Oct 3, 2022. "
+        "Migrate to an alternative flow: gspread.auth.local_server_flow"
+        "see: https://developers.googleblog.com/2022/02/making-oauth-flows-safer.html?m=1#disallowed-oob"
+    )
     flow = InstalledAppFlow.from_client_config(client_config, scopes)
     return flow.run_console()
 
 
 def load_credentials(filename=DEFAULT_AUTHORIZED_USER_FILENAME):
-    if os.path.exists(filename):
+    if filename.exists():
         return Credentials.from_authorized_user_file(filename)
 
     return None
 
 
 def store_credentials(creds, filename=DEFAULT_AUTHORIZED_USER_FILENAME, strip="token"):
-    os.makedirs(os.path.dirname(filename), exist_ok=True)
-    with open(filename, "w") as f:
+    filename.parent.mkdir(parents=True, exist_ok=True)
+    with filename.open("w") as f:
         f.write(creds.to_json(strip))
 
 
@@ -95,11 +119,12 @@ def oauth(
     flow=local_server_flow,
     credentials_filename=DEFAULT_CREDENTIALS_FILENAME,
     authorized_user_filename=DEFAULT_AUTHORIZED_USER_FILENAME,
+    client_factory=Client,
 ):
     r"""Authenticate with OAuth Client ID.
 
     By default this function will use the local server strategy and open
-    the authorization URL in the user’s browser::
+    the authorization URL in the user's browser::
 
         gc = gspread.oauth()
 
@@ -143,26 +168,32 @@ def oauth(
     :param str credentials_filename: Filepath (including name) pointing to a
         credentials `.json` file.
         Defaults to DEFAULT_CREDENTIALS_FILENAME:
+
             * `%APPDATA%\gspread\credentials.json` on Windows
             * `~/.config/gspread/credentials.json` everywhere else
     :param str authorized_user_filename: Filepath (including name) pointing to
         an authorized user `.json` file.
         Defaults to DEFAULT_AUTHORIZED_USER_FILENAME:
+
             * `%APPDATA%\gspread\authorized_user.json` on Windows
             * `~/.config/gspread/authorized_user.json` everywhere else
+    :type client_factory: :class:`gspread.ClientFactory`
+    :param client_factory: A factory function that returns a client class.
+        Defaults to :class:`gspread.Client` (but could also use
+        :class:`gspread.BackoffClient` to avoid rate limiting)
 
-    :rtype: :class:`gspread.Client`
+    :rtype: :class:`gspread.client.Client`
     """
+    authorized_user_filename = Path(authorized_user_filename)
     creds = load_credentials(filename=authorized_user_filename)
 
-    if not creds:
+    if not type(creds) is Credentials:
         with open(credentials_filename) as json_file:
             client_config = json.load(json_file)
         creds = flow(client_config=client_config, scopes=scopes)
         store_credentials(creds, filename=authorized_user_filename)
 
-    client = Client(auth=creds)
-    return client
+    return client_factory(auth=creds)
 
 
 def oauth_from_dict(
@@ -170,20 +201,21 @@ def oauth_from_dict(
     authorized_user_info=None,
     scopes=DEFAULT_SCOPES,
     flow=local_server_flow,
+    client_factory=Client,
 ):
     r"""Authenticate with OAuth Client ID.
 
     By default this function will use the local server strategy and open
     the authorization URL in the user's browser::
 
-        gc = gspread.oauth()
+        gc = gspread.oauth_from_dict()
 
     Another option is to run a console strategy. This way, the user is
     instructed to open the authorization URL in their browser. Once the
     authorization is complete, the user must then copy & paste the
     authorization code into the application::
 
-        gc = gspread.oauth(flow=gspread.auth.console_flow)
+        gc = gspread.oauth_from_dict(flow=gspread.auth.console_flow)
 
 
     ``scopes`` parameter defaults to read/write scope available in
@@ -199,7 +231,7 @@ def oauth_from_dict(
     Obviously any method of ``gspread`` that updates a spreadsheet
     **will not work** in this case::
 
-        gc = gspread.oauth(scopes=gspread.auth.READONLY_SCOPES)
+        gc = gspread.oauth_from_dict(scopes=gspread.auth.READONLY_SCOPES)
 
         sh = gc.open("A spreadsheet")
         sh.sheet1.update('A1', '42')   # <-- this will not work
@@ -209,9 +241,14 @@ def oauth_from_dict(
     the authenticated user info, this can be passed again to authenticate
     the user without the need to run the flow again.
 
-        gc = gspread.oauth(
-            credentials=my_creds,
-            authorized_user_info=my_auth_user,
+    ..
+        code block below must be explicitly announced using code-block
+
+    .. code-block:: python
+
+        gc = gspread.oauth_from_dict(
+                credentials=my_creds,
+                authorized_user_info=my_auth_user
         )
 
     :param dict credentials: The credentials from google cloud platform
@@ -220,8 +257,12 @@ def oauth_from_dict(
     :param list scopes: The scopes used to obtain authorization.
     :param function flow: OAuth flow to use for authentication.
         Defaults to :meth:`~gspread.auth.local_server_flow`
+    :type client_factory: :class:`gspread.ClientFactory`
+    :param client_factory: A factory function that returns a client class.
+        Defaults to :class:`gspread.Client` (but could also use
+        :class:`gspread.BackoffClient` to avoid rate limiting)
 
-    :rtype: :class:`gspread.Client`
+    :rtype: (`gspread.client.Client`, str)
     """
 
     creds = None
@@ -231,7 +272,7 @@ def oauth_from_dict(
     if not creds:
         creds = flow(client_config=credentials, scopes=scopes)
 
-    client = Client(auth=creds)
+    client = client_factory(auth=creds)
 
     # must return the creds to the user
     # must strip the token an use the dedicated method from Credentials
@@ -239,7 +280,11 @@ def oauth_from_dict(
     return (client, creds.to_json("token"))
 
 
-def service_account(filename=DEFAULT_SERVICE_ACCOUNT_FILENAME, scopes=DEFAULT_SCOPES):
+def service_account(
+    filename=DEFAULT_SERVICE_ACCOUNT_FILENAME,
+    scopes=DEFAULT_SCOPES,
+    client_factory=Client,
+):
     """Authenticate using a service account.
 
     ``scopes`` parameter defaults to read/write scope available in
@@ -257,14 +302,18 @@ def service_account(filename=DEFAULT_SERVICE_ACCOUNT_FILENAME, scopes=DEFAULT_SC
 
     :param str filename: The path to the service account json file.
     :param list scopes: The scopes used to obtain authorization.
+    :type client_factory: :class:`gspread.ClientFactory`
+    :param client_factory: A factory function that returns a client class.
+        Defaults to :class:`gspread.Client` (but could also use
+        :class:`gspread.BackoffClient` to avoid rate limiting)
 
-    :rtype: :class:`gspread.Client`
+    :rtype: :class:`gspread.client.Client`
     """
     creds = ServiceAccountCredentials.from_service_account_file(filename, scopes=scopes)
-    return Client(auth=creds)
+    return client_factory(auth=creds)
 
 
-def service_account_from_dict(info, scopes=DEFAULT_SCOPES):
+def service_account_from_dict(info, scopes=DEFAULT_SCOPES, client_factory=Client):
     """Authenticate using a service account (json).
 
     ``scopes`` parameter defaults to read/write scope available in
@@ -282,11 +331,15 @@ def service_account_from_dict(info, scopes=DEFAULT_SCOPES):
 
     :param info (Mapping[str, str]): The service account info in Google format
     :param list scopes: The scopes used to obtain authorization.
+    :type client_factory: :class:`gspread.ClientFactory`
+    :param client_factory: A factory function that returns a client class.
+        Defaults to :class:`gspread.Client` (but could also use
+        :class:`gspread.BackoffClient` to avoid rate limiting)
 
-    :rtype: :class:`gspread.Client`
+    :rtype: :class:`gspread.client.Client`
     """
     creds = ServiceAccountCredentials.from_service_account_info(
         info=info,
         scopes=scopes,
     )
-    return Client(auth=creds)
+    return client_factory(auth=creds)
