@@ -8,22 +8,17 @@ This module contains utility functions.
 
 import re
 from collections import defaultdict, namedtuple
+from collections.abc import Sequence
 from functools import wraps
-from math import inf
-
-try:
-    from collections.abc import Sequence
-except ImportError:
-    from collections import Sequence
-
 from itertools import chain
+from math import inf
 from urllib.parse import quote as uquote
 
 from google.auth.credentials import Credentials as Credentials
 from google.oauth2.credentials import Credentials as UserCredentials
 from google.oauth2.service_account import Credentials as ServiceAccountCredentials
 
-from .exceptions import IncorrectCellLabel, NoValidUrlKeyFound
+from .exceptions import IncorrectCellLabel, InvalidInputValue, NoValidUrlKeyFound
 
 MAGIC_NUMBER = 64
 CELL_ADDR_RE = re.compile(r"([A-Za-z]+)([1-9]\d*)")
@@ -39,6 +34,28 @@ ValueRenderOption = namedtuple(
 ValueInputOption = namedtuple("_ValueInputOption", ["raw", "user_entered"])(
     "RAW", "USER_ENTERED"
 )
+MimeType = namedtuple(
+    "_MimeType",
+    ["google_sheets", "pdf", "excel", "csv", "open_office_sheet", "tsv", "zip"],
+)(
+    "application/vnd.google-apps.spreadsheet",
+    "application/pdf",
+    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
+    "text/csv",
+    "application/vnd.oasis.opendocument.spreadsheet",
+    "text/tab-separated-values",
+    "application/zip",
+)
+ExportFormat = namedtuple(
+    "_ExportFormat", ["PDF", "EXCEL", "CSV", "OPEN_OFFICE_SHEET", "TSV", "ZIPPED_HTML"]
+)(
+    MimeType.pdf,
+    MimeType.excel,
+    MimeType.csv,
+    MimeType.open_office_sheet,
+    MimeType.tsv,
+    MimeType.zip,
+)
 
 
 def convert_credentials(credentials):
@@ -95,6 +112,7 @@ def numericise(
     allow_underscores_in_numeric_literals=False,
 ):
     """Returns a value that depends on the input:
+
         - Float if input is a string that can be converted to Float
         - Integer if input is a string that can be converted to integer
         - Zero if the input is a string that is empty and empty2zero flag is set
@@ -102,28 +120,39 @@ def numericise(
 
     Examples::
 
-    >>> numericise("faa")
-    'faa'
+        >>> numericise("faa")
+        'faa'
+
     >>> numericise("3")
     3
+
     >>> numericise("3_2", allow_underscores_in_numeric_literals=False)
     '3_2'
+
     >>> numericise("3_2", allow_underscores_in_numeric_literals=True)
     32
+
     >>> numericise("3.1")
     3.1
+
     >>> numericise("2,000.1")
     2000.1
+
     >>> numericise("", empty2zero=True)
     0
+
     >>> numericise("", empty2zero=False)
     ''
+
     >>> numericise("", default_blank=None)
     >>>
+
     >>> numericise("", default_blank="foo")
     'foo'
+
     >>> numericise("")
     ''
+
     >>> numericise(None)
     >>>
     """
@@ -153,16 +182,16 @@ def numericise(
 
 
 def numericise_all(
-    input,
+    values,
     empty2zero=False,
     default_blank="",
     allow_underscores_in_numeric_literals=False,
-    ignore=None,
+    ignore=[],
 ):
     """Returns a list of numericised values from strings except those from the
     row specified as ignore.
 
-    :param list input: Input row
+    :param list values: Input row
     :param bool empty2zero: (optional) Whether or not to return empty cells
         as 0 (zero). Defaults to ``False``.
     :param str default_blank: Which value to use for blank cells,
@@ -172,18 +201,21 @@ def numericise_all(
     :param list ignore: List of ints of indices of the row (index 1) to ignore
         numericising.
     """
-    ignored_rows = [input[x - 1] for x in (ignore or [])]
+    # in case someone explicitly passes `None` as ignored list
+    ignore = ignore or []
+
     numericised_list = [
-        s
-        if s in ignored_rows
+        values[index]
+        if index + 1 in ignore
         else numericise(
-            s,
+            values[index],
             empty2zero=empty2zero,
             default_blank=default_blank,
             allow_underscores_in_numeric_literals=allow_underscores_in_numeric_literals,
         )
-        for s in input
+        for index in range(len(values))
     ]
+
     return numericised_list
 
 
@@ -249,7 +281,7 @@ def a1_to_rowcol(label):
 
         col = 0
         for i, c in enumerate(reversed(column_label)):
-            col += (ord(c) - MAGIC_NUMBER) * (26 ** i)
+            col += (ord(c) - MAGIC_NUMBER) * (26**i)
     else:
         raise IncorrectCellLabel(label)
 
@@ -302,7 +334,7 @@ def _a1_to_rowcol_unbounded(label):
         if column_label:
             col = 0
             for i, c in enumerate(reversed(column_label.upper())):
-                col += (ord(c) - MAGIC_NUMBER) * (26 ** i)
+                col += (ord(c) - MAGIC_NUMBER) * (26**i)
         else:
             col = inf
 
@@ -329,8 +361,8 @@ def a1_range_to_grid_range(name, sheet_id=None):
 
     Examples::
 
-    >>> a1_range_to_grid_range('A1:A1')
-    {'startRowIndex': 0, 'endRowIndex': 1, 'startColumnIndex': 0, 'endColumnIndex': 1}
+        >>> a1_range_to_grid_range('A1:A1')
+        {'startRowIndex': 0, 'endRowIndex': 1, 'startColumnIndex': 0, 'endColumnIndex': 1}
 
     >>> a1_range_to_grid_range('A3:B4')
     {'startRowIndex': 2, 'endRowIndex': 4, 'startColumnIndex': 0, 'endColumnIndex': 2}
@@ -380,6 +412,48 @@ def a1_range_to_grid_range(name, sheet_id=None):
     return grid_range
 
 
+def column_letter_to_index(column):
+    """Converts a column letter to its numerical index.
+
+    This is useful when using the method :meth:`gspread.worksheet.Worksheet.col_values`.
+    Which requires a column index.
+
+    This function is case-insensitive.
+
+    Raises :exc:`gspread.exceptions.InvalidInputValue` in case of invalid input.
+
+    Examples::
+
+        >>> column_letter_to_index("a")
+        1
+
+    >>> column_letter_to_index("A")
+    1
+
+    >>> column_letter_to_index("AZ")
+    52
+
+    >>> column_letter_to_index("!@#$%^&")
+    ...
+    gspread.exceptions.InvalidInputValue: invalid value: !@#$%^&, must be a column letter
+    """
+    try:
+        (_, index) = _a1_to_rowcol_unbounded(column)
+    except IncorrectCellLabel:
+        # make it coherent and raise the same exception in case of any error
+        # from user input value
+        raise InvalidInputValue(
+            "invalid value: {}, must be a column letter".format(column)
+        )
+
+    if index is inf:
+        raise InvalidInputValue(
+            "invalid value: {}, must be a column letter".format(column)
+        )
+
+    return index
+
+
 def cast_to_a1_notation(method):
     """Decorator function casts wrapped arguments to A1 notation in range
     method calls.
